@@ -8,6 +8,15 @@ from copy import deepcopy
 
 from ctapipe.io import SimTelEventSource
 from ctapipe.utils import get_dataset_path
+from ctapipe.instrument import CameraGeometry
+
+
+@pytest.fixture(scope="session")
+def camera_geometries():
+    return [
+        CameraGeometry.from_name(name)
+        for name in ["LSTCam", "NectarCam", "CHEC", "FlashCam", "MAGICCam"]
+    ]
 
 
 @pytest.fixture(scope="session")
@@ -47,7 +56,7 @@ def example_event(_global_example_event):
 
     .. code-block::
         def test_my_thing(test_event):
-            assert len(test_event.r0.tels_with_data)>0
+            assert len(test_event.r0.tel) > 0
 
     """
     return deepcopy(_global_example_event)
