@@ -11,44 +11,42 @@ import numpy as np
 from .core import Container, Field, Map
 
 __all__ = [
-    "R0Container",
-    "R0CameraContainer",
-    "R1Container",
-    "R1CameraContainer",
-    "DL0Container",
-    "DL0CameraContainer",
-    "DL1Container",
-    "DL1CameraContainer",
-    "MCDL1CameraContainer",
-    "EventCameraCalibrationContainer",
-    "EventCalibrationContainer",
-    "MCEventContainer",
-    "MCHeaderContainer",
-    "MCCameraEventContainer",
-    "DL1CameraCalibrationContainer",
-    "TriggerContainer",
-    "ReconstructedContainer",
-    "ReconstructedShowerContainer",
-    "ReconstructedEnergyContainer",
-    "ParticleClassificationContainer",
-    "DataContainer",
-    "HillasParametersContainer",
-    "LeakageContainer",
+    "ArrayEventContainer",
     "ConcentrationContainer",
-    "MorphologyContainer",
-    "TimingParametersContainer",
+    "DL0CameraContainer",
+    "DL0Container",
+    "DL1CameraCalibrationContainer",
+    "DL1CameraContainer",
+    "DL1Container",
+    "EventCalibrationContainer",
+    "EventCameraCalibrationContainer",
+    "EventIndexContainer",
+    "EventType",
     "FlatFieldContainer",
-    "PedestalContainer",
-    "PixelStatusContainer",
-    "WaveformCalibrationContainer",
+    "HillasParametersContainer",
+    "ImageParametersContainer",
+    "LeakageContainer",
     "MonitoringCameraContainer",
     "MonitoringContainer",
-    "EventAndMonDataContainer",
-    "EventIndexContainer",
-    "TelEventIndexContainer",
-    "ImageParametersContainer",
+    "MorphologyContainer",
+    "ParticleClassificationContainer",
+    "PedestalContainer",
+    "PixelStatusContainer",
+    "R0CameraContainer",
+    "R0Container",
+    "R1CameraContainer",
+    "R1Container",
+    "ReconstructedContainer",
+    "ReconstructedEnergyContainer",
+    "ReconstructedShowerContainer",
+    "SimulatedCameraContainer",
+    "SimulatedShowerContainer",
     "SimulatedShowerDistribution",
-    "EventType",
+    "SimulationConfigContainer",
+    "TelEventIndexContainer",
+    "TimingParametersContainer",
+    "TriggerContainer",
+    "WaveformCalibrationContainer",
 ]
 
 
@@ -57,8 +55,8 @@ NAN_TIME = Time(np.ma.masked_array(nan, mask=True), format="mjd")
 
 
 class EventType(enum.Enum):
-    """ These numbers come from  the document *CTA R1/Event Data Model Specification*
-    version 1 revision C.  They may be updated in future revisions """
+    """These numbers come from  the document *CTA R1/Event Data Model Specification*
+    version 1 revision C.  They may be updated in future revisions"""
 
     # calibrations are 0-15
     FLATFIELD = 0
@@ -112,7 +110,9 @@ class HillasParametersContainer(Container):
     phi = Field(nan * u.deg, "polar coordinate of centroid", unit=u.deg)
 
     length = Field(nan * u.m, "standard deviation along the major-axis", unit=u.m)
+    length_uncertainty = Field(nan * u.m, "uncertainty of length", unit=u.m)
     width = Field(nan * u.m, "standard spread along the minor-axis", unit=u.m)
+    width_uncertainty = Field(nan * u.m, "uncertainty of width", unit=u.m)
     psi = Field(nan * u.deg, "rotation angle of ellipse", unit=u.deg)
 
     skewness = Field(nan, "measure of the asymmetry")
@@ -169,9 +169,7 @@ class TimingParametersContainer(Container):
     slope = Field(
         nan / u.m, "Slope of arrival times along main shower axis", unit=1 / u.m
     )
-    slope_err = Field(nan / u.m, "Uncertainty `slope`", unit=1 / u.m)
     intercept = Field(nan, "intercept of arrival times along main shower axis")
-    intercept_err = Field(nan, "Uncertainty `intercept`")
     deviation = Field(
         nan,
         "Root-mean-square deviation of the pulse times "
@@ -256,23 +254,6 @@ class DL1CameraContainer(Container):
     parameters = Field(ImageParametersContainer(), "Parameters derived from images")
 
 
-class MCDL1CameraContainer(Container):
-    """ Contains all fields of the DL1CameraContainer, but adds fields for simulated
-    DL1 image information."""
-
-    true_image = Field(
-        None,
-        "Numpy array of camera image in PE as simulated before noise has been added. "
-        "Shape: (n_pixel)",
-        dtype=np.float32,
-        ndim=1,
-    )
-
-    true_parameters = Field(
-        ImageParametersContainer(), "Parameters derived from the true_image"
-    )
-
-
 class DL1Container(Container):
     """ DL1 Calibrated Camera Images and associated data"""
 
@@ -285,9 +266,10 @@ class DL1CameraCalibrationContainer(Container):
     """
 
     pedestal_offset = Field(
-        0,
-        "Additive coefficients for the pedestal calibration of extracted charge "
-        "for each pixel",
+        None,
+        "Residual mean pedestal of the waveforms for each pixel."
+        " This value is subtracted from the waveforms of each pixel before"
+        " the pulse extraction.",
     )
     absolute_factor = Field(
         1,
@@ -301,7 +283,7 @@ class DL1CameraCalibrationContainer(Container):
         "uniform illumination.",
     )
     time_shift = Field(
-        0,
+        None,
         "Additive coefficients for the timing correction before charge extraction "
         "for each pixel",
     )
@@ -312,12 +294,6 @@ class R0CameraContainer(Container):
     Storage of raw data from a single telescope
     """
 
-    trigger_time = Field(
-        None, "Telescope trigger time, start of waveform " "readout, None for MCs"
-    )
-    trigger_type = Field(0o0, "camera's event trigger type if applicable")
-    num_trig_pix = Field(0, "Number of trigger groups (sectors) listed")
-    trig_pix_id = Field(None, "pixels involved in the camera trigger")
     waveform = Field(
         None, ("numpy array containing ADC samples" "(n_channels, n_pixels, n_samples)")
     )
@@ -328,7 +304,6 @@ class R0Container(Container):
     Storage of a Merged Raw Data Event
     """
 
-    tels_with_data = Field([], "set of tel_ids for telescopes with data")
     tel = Field(Map(R0CameraContainer), "map of tel_id to R0CameraContainer")
 
 
@@ -337,9 +312,6 @@ class R1CameraContainer(Container):
     Storage of r1 calibrated data from a single telescope
     """
 
-    trigger_time = Field(None, "Telescope trigger time, start of waveform " "readout")
-    trigger_type = Field(0o0, "camera trigger type")
-
     waveform = Field(
         None,
         (
@@ -361,7 +333,6 @@ class R1Container(Container):
     Storage of a r1 calibrated Data Event
     """
 
-    tels_with_data = Field([], "set of tel_ids for telescopes with data")
     tel = Field(Map(R1CameraContainer), "map of tel_id to R1CameraContainer")
 
 
@@ -370,9 +341,6 @@ class DL0CameraContainer(Container):
     Storage of data volume reduced dl0 data from a single telescope
     """
 
-    trigger_time = Field(None, "Telescope trigger time, start of waveform " "readout")
-    trigger_type = Field(0o0, "camera trigger type")
-
     waveform = Field(
         None,
         (
@@ -397,81 +365,66 @@ class DL0Container(Container):
     Storage of a data volume reduced Event
     """
 
-    tels_with_data = Field([], "set of tel_ids for telescopes with data")
     tel = Field(Map(DL0CameraContainer), "map of tel_id to DL0CameraContainer")
 
 
-class MCCameraEventContainer(Container):
+class SimulatedShowerContainer(Container):
+    container_prefix = "true"
+    energy = Field(nan * u.TeV, "Simulated Energy", unit=u.TeV)
+    alt = Field(nan * u.deg, "Simulated altitude", unit=u.deg)
+    az = Field(nan * u.deg, "Simulated azimuth", unit=u.deg)
+    core_x = Field(nan * u.m, "Simulated core position (x)", unit=u.m)
+    core_y = Field(nan * u.m, "Simulated core position (y)", unit=u.m)
+    h_first_int = Field(nan * u.m, "Height of first interaction", unit=u.m)
+    x_max = Field(
+        nan * u.g / (u.cm ** 2), "Simulated Xmax value", unit=u.g / (u.cm ** 2)
+    )
+    shower_primary_id = Field(
+        -1,
+        "Simulated shower primary ID 0 (gamma), 1(e-),"
+        "2(mu-), 100*A+Z for nucleons and nuclei,"
+        "negative for antimatter.",
+    )
+
+
+class SimulatedCameraContainer(Container):
     """
-    Storage of mc data for a single telescope that change per event
+    True images and parameters derived from them, analgous to the `DL1CameraContainer`
+    but for simulated data.
     """
 
+    container_prefix = ""
+
     true_image = Field(
         None,
         "Numpy array of camera image in PE as simulated before noise has been added. "
         "Shape: (n_pixel)",
+        dtype=np.float32,
+        ndim=1,
     )
 
-    # TODO: should move dc_to_pe and pedestal to a MC Monitoring Container,
-    # so they are not written for each event.
-    dc_to_pe = Field(None, "DC/PE calibration arrays from MC file")
-    pedestal = Field(None, "pedestal calibration arrays from MC file")
-    azimuth_raw = Field(0, "Raw azimuth angle [radians from N->E] for the telescope")
-    altitude_raw = Field(0, "Raw altitude angle [radians] for the telescope")
-    azimuth_cor = Field(
-        0, "the tracking Azimuth corrected for pointing errors for the telescope"
-    )
-    altitude_cor = Field(
-        0, "the tracking Altitude corrected for pointing errors for the telescope"
+    true_parameters = Field(
+        ImageParametersContainer(), "Parameters derived from the true_image"
     )
 
 
-class MCEventContainer(Container):
+class SimulatedEventContainer(Container):
+    shower = Field(SimulatedShowerContainer(), "True event information")
+    tel = Field(Map(SimulatedCameraContainer))
+
+
+class SimulationConfigContainer(Container):
     """
-    Monte-Carlo
+    Configuration parameters of the simulation
     """
 
-    container_prefix = "true"
-
-    energy = Field(nan * u.TeV, "Monte-Carlo Energy", unit=u.TeV)
-    alt = Field(nan * u.deg, "Monte-carlo altitude", unit=u.deg)
-    az = Field(nan * u.deg, "Monte-Carlo azimuth", unit=u.deg)
-    core_x = Field(nan * u.m, "MC core position", unit=u.m)
-    core_y = Field(nan * u.m, "MC core position", unit=u.m)
-    h_first_int = Field(nan * u.m, "Height of first interaction", unit=u.m)
-    x_max = Field(nan * u.g / (u.cm ** 2), "MC Xmax value", unit=u.g / (u.cm ** 2))
-    shower_primary_id = Field(
-        -1,
-        "MC shower primary ID 0 (gamma), 1(e-),"
-        "2(mu-), 100*A+Z for nucleons and nuclei,"
-        "negative for antimatter.",
-    )
-    tel = Field(Map(MCCameraEventContainer), "map of tel_id to MCCameraEventContainer")
-
-
-class MCHeaderContainer(Container):
-    """
-    Monte-Carlo information that doesn't change per event
-    """
-
-    run_array_direction = Field(
-        [],
-        (
-            "the tracking/pointing direction in "
-            "[radians]. Depending on 'tracking_mode' "
-            "this either contains: "
-            "[0]=Azimuth, [1]=Altitude in mode 0, "
-            "OR "
-            "[0]=R.A., [1]=Declination in mode 1."
-        ),
-    )
     corsika_version = Field(nan, "CORSIKA version * 1000")
     simtel_version = Field(nan, "sim_telarray version * 1000")
     energy_range_min = Field(
-        nan * u.TeV, "Lower limit of energy range " "of primary particle", unit=u.TeV
+        nan * u.TeV, "Lower limit of energy range of primary particle", unit=u.TeV
     )
     energy_range_max = Field(
-        nan * u.TeV, "Upper limit of energy range " "of primary particle", unit=u.TeV
+        nan * u.TeV, "Upper limit of energy range of primary particle", unit=u.TeV
     )
     prod_site_B_total = Field(nan * u.uT, "total geomagnetic field", unit=u.uT)
     prod_site_B_declination = Field(nan * u.rad, "magnetic declination", unit=u.rad)
@@ -479,9 +432,7 @@ class MCHeaderContainer(Container):
     prod_site_alt = Field(nan * u.m, "height of observation level", unit=u.m)
     spectral_index = Field(nan, "Power-law spectral index of spectrum")
     shower_prog_start = Field(
-        nan,
-        """Time when shower simulation started,
-                              CORSIKA: only date""",
+        nan, "Time when shower simulation started, CORSIKA: only date"
     )
     shower_prog_id = Field(nan, "CORSIKA=1, ALTAI=2, KASCADE=3, MOCCA=4")
     detector_prog_start = Field(nan, "Time when detector simulation started")
@@ -497,30 +448,44 @@ class MCHeaderContainer(Container):
     min_viewcone_radius = Field(nan * u.deg, "Minimum viewcone radius", unit=u.deg)
     max_scatter_range = Field(nan * u.m, "Maximum scatter range", unit=u.m)
     min_scatter_range = Field(nan * u.m, "Minimum scatter range", unit=u.m)
-    core_pos_mode = Field(nan, "Core Position Mode (fixed/circular/...)")
+    core_pos_mode = Field(nan, "Core Position Mode (0=Circular, 1=Rectangular)")
     injection_height = Field(nan * u.m, "Height of particle injection", unit=u.m)
     atmosphere = Field(nan * u.m, "Atmospheric model number")
-    corsika_iact_options = Field(nan, "Detector MC information")
-    corsika_low_E_model = Field(nan, "Detector MC information")
-    corsika_high_E_model = Field(nan, "Detector MC information")
-    corsika_bunchsize = Field(nan, "Number of photons per bunch")
+    corsika_iact_options = Field(nan, "CORSIKA simulation options for IACTs")
+    corsika_low_E_model = Field(nan, "CORSIKA low-energy simulation physics model")
+    corsika_high_E_model = Field(
+        nan,
+        "CORSIKA physics model ID for high energies "
+        "(1=VENUS, 2=SIBYLL, 3=QGSJET, 4=DPMJET, 5=NeXus, 6=EPOS) ",
+    )
+    corsika_bunchsize = Field(nan, "Number of Cherenkov photons per bunch")
     corsika_wlen_min = Field(
         nan * u.m, "Minimum wavelength of cherenkov light", unit=u.nm
     )
     corsika_wlen_max = Field(
         nan * u.m, "Maximum wavelength of cherenkov light", unit=u.nm
     )
-    corsika_low_E_detail = Field(nan, "Detector MC information")
-    corsika_high_E_detail = Field(nan, "Detector MC information")
+    corsika_low_E_detail = Field(
+        nan, "More details on low E interaction model (version etc.)"
+    )
+    corsika_high_E_detail = Field(
+        nan, "More details on high E interaction model (version etc.)"
+    )
 
 
 class TelescopeTriggerContainer(Container):
+    container_prefix = ""
     time = Field(NAN_TIME, "Telescope trigger time")
+    n_trigger_pixels = Field(-1, "Number of trigger groups (sectors) listed")
+    trigger_pixels = Field(None, "pixels involved in the camera trigger")
 
 
 class TriggerContainer(Container):
+    container_prefix = ""
     time = Field(NAN_TIME, "central average time stamp")
-    tels_with_trigger = Field([], "list of telescope ids with data")
+    tels_with_trigger = Field(
+        [], "List of telescope ids that triggered the array event"
+    )
     event_type = Field(EventType.SUBARRAY, "Event type")
     tel = Field(Map(TelescopeTriggerContainer), "telescope-wise trigger information")
 
@@ -674,8 +639,6 @@ class EventCalibrationContainer(Container):
     Container for calibration coefficients for the current event
     """
 
-    tels_with_data = Field([], "set of tel_ids for telescopes with data")
-
     # create the camera container
     tel = Field(
         Map(EventCameraCalibrationContainer),
@@ -683,26 +646,6 @@ class EventCalibrationContainer(Container):
     )
 
 
-class DataContainer(Container):
-    """ Top-level container for all event information """
-
-    index = Field(EventIndexContainer(), "event indexing information")
-    r0 = Field(R0Container(), "Raw Data")
-    r1 = Field(R1Container(), "R1 Calibrated Data")
-    dl0 = Field(DL0Container(), "DL0 Data Volume Reduced Data")
-    dl1 = Field(DL1Container(), "DL1 Calibrated image")
-    dl2 = Field(ReconstructedContainer(), "Reconstructed Shower Information")
-    mc = Field(MCEventContainer(), "Monte-Carlo data")
-    mcheader = Field(MCHeaderContainer(), "Monte-Carlo run header data")
-    trigger = Field(TriggerContainer(), "central trigger information")
-    count = Field(0, "number of events processed")
-    pointing = Field(PointingContainer(), "Array and telescope pointing positions")
-    calibration = Field(
-        EventCalibrationContainer(),
-        "Container for calibration coefficients for the current event",
-    )
-
-
 class MuonRingContainer(Container):
     """Container for the result of a ring fit, center_x, center_y"""
 
@@ -718,10 +661,10 @@ class MuonRingContainer(Container):
 
 
 class MuonEfficiencyContainer(Container):
-    width = Field(nan, "width of the muon ring in degrees")
-    impact = Field(nan, "distance of muon impact position from center of mirror")
-    impact_x = Field(nan, "impact parameter x position")
-    impact_y = Field(nan, "impact parameter y position")
+    width = Field(nan * u.deg, "width of the muon ring in degrees")
+    impact = Field(nan * u.m, "distance of muon impact position from center of mirror")
+    impact_x = Field(nan * u.m, "impact parameter x position")
+    impact_y = Field(nan * u.m, "impact parameter y position")
     optical_efficiency = Field(nan, "optical efficiency muon")
 
 
@@ -854,10 +797,10 @@ class WaveformCalibrationContainer(Container):
 
     time = Field(nan * u.s, "Time associated to the calibration event", unit=u.s)
     time_min = Field(
-        nan * u.s, "Earliest time of validity for the calibration event", unit=u.s,
+        nan * u.s, "Earliest time of validity for the calibration event", unit=u.s
     )
     time_max = Field(
-        nan * u.s, "Latest time of validity for the calibration event", unit=u.s,
+        nan * u.s, "Latest time of validity for the calibration event", unit=u.s
     )
 
     dc_to_pe = Field(
@@ -902,22 +845,12 @@ class MonitoringContainer(Container):
     Root container for monitoring data (MON)
     """
 
-    tels_with_data = Field([], "list of telescopes with data")
-
     # create the camera container
     tel = Field(
         Map(MonitoringCameraContainer), "map of tel_id to MonitoringCameraContainer"
     )
 
 
-class EventAndMonDataContainer(DataContainer):
-    """
-    Data container including monitoring information
-    """
-
-    mon = Field(MonitoringContainer(), "container for monitoring data (MON)")
-
-
 class SimulatedShowerDistribution(Container):
     """
     2D histogram of simulated number of showers simulated as function of energy and
@@ -936,3 +869,23 @@ class SimulatedShowerDistribution(Container):
         None, "array of core-distance bin lower edges, as in np.histogram", unit=u.m
     )
     histogram = Field(None, "array of histogram entries, size (n_bins_x, n_bins_y)")
+
+
+class ArrayEventContainer(Container):
+    """ Top-level container for all event information """
+
+    index = Field(EventIndexContainer(), "event indexing information")
+    r0 = Field(R0Container(), "Raw Data")
+    r1 = Field(R1Container(), "R1 Calibrated Data")
+    dl0 = Field(DL0Container(), "DL0 Data Volume Reduced Data")
+    dl1 = Field(DL1Container(), "DL1 Calibrated image")
+    dl2 = Field(ReconstructedContainer(), "Reconstructed Shower Information")
+    simulation = Field(SimulatedEventContainer(), "Simulated Event Information")
+    trigger = Field(TriggerContainer(), "central trigger information")
+    count = Field(0, "number of events processed")
+    pointing = Field(PointingContainer(), "Array and telescope pointing positions")
+    calibration = Field(
+        EventCalibrationContainer(),
+        "Container for calibration coefficients for the current event",
+    )
+    mon = Field(MonitoringContainer(), "container for event-wise monitoring data (MON)")
