@@ -1,6 +1,6 @@
 from ctapipe.image.tests.test_hillas import create_sample_image
 from ctapipe.image.hillas import hillas_parameters
-from ctapipe.image.concentration import concentration
+from ctapipe.image.concentration import concentration_parameters
 import astropy.units as u
 import pytest
 
@@ -10,7 +10,7 @@ def test_concentration():
 
     hillas = hillas_parameters(geom[clean_mask], image[clean_mask])
 
-    conc = concentration(geom, image, hillas)
+    conc = concentration_parameters(geom, image, hillas)
 
     assert 0.1 <= conc.cog <= 0.3
     assert 0.05 <= conc.pixel <= 0.2
@@ -24,7 +24,7 @@ def test_width_0():
     hillas = hillas_parameters(geom[clean_mask], image[clean_mask])
     hillas.width = 0 * u.m
 
-    conc = concentration(geom, image, hillas)
+    conc = concentration_parameters(geom, image, hillas)
     assert conc.core == 0
 
 
