@@ -3,24 +3,58 @@
 Utilities for reading or working with Camera geometry files
 """
 import logging
+import warnings
+from typing import TypeVar
 
 import numpy as np
 from astropy import units as u
 from astropy.coordinates import Angle, SkyCoord
+from astropy.coordinates import BaseCoordinateFrame
 from astropy.table import Table
 from astropy.utils import lazyproperty
-from scipy.spatial import cKDTree as KDTree
 from scipy.sparse import lil_matrix, csr_matrix
-import warnings
+from scipy.spatial import cKDTree as KDTree
 
+from ctapipe.coordinates import CameraFrame
 from ctapipe.utils import get_table_dataset
 from ctapipe.utils.linalg import rotation_matrix_2d
-from ctapipe.coordinates import CameraFrame
-
+from enum import Enum, unique
 
 __all__ = ["CameraGeometry", "UnknownPixelShapeWarning"]
 
 logger = logging.getLogger(__name__)
+CG = TypeVar("CG", bound="CameraGeometry")  # for forward-referencing type hints
+
+
+@unique
+class PixelShape(Enum):
+    CIRCLE = "circle"
+    SQUARE = "square"
+    HEXAGON = "hexagon"
+
+    @classmethod
+    def from_string(cls, name):
+        name = name.lower()
+
+        if name.startswith("hex"):
+            return cls.HEXAGON
+
+        if name.startswith("rect") or name == "square":
+            return cls.SQUARE
+
+        if name.startswith("circ"):
+            return cls.CIRCLE
+
+        raise TypeError(f"Unknown pixel shape {name}")
+
+
+#: mapper from simtel pixel shape integerrs to our shape and rotation angle
+SIMTEL_PIXEL_SHAPES = {
+    0: (PixelShape.CIRCLE, Angle(0, u.deg)),
+    1: (PixelShape.HEXAGON, Angle(0, u.deg)),
+    2: (PixelShape.SQUARE, Angle(0, u.deg)),
+    3: (PixelShape.HEXAGON, Angle(30, u.deg)),
+}
 
 
 class CameraGeometry:
@@ -88,6 +122,14 @@ class CameraGeometry:
             )
 
         assert len(pix_x) == len(pix_y), "pix_x and pix_y must have same length"
+
+        if isinstance(pix_type, str):
+            pix_type = PixelShape.from_string(pix_type)
+        elif not isinstance(pix_type, PixelShape):
+            raise TypeError(
+                f"pix_type most be a PixelShape or the name of a PixelShape, got {pix_type}"
+            )
+
         self.n_pixels = len(pix_x)
         self.camera_name = camera_name
         self.pix_id = pix_id
@@ -134,7 +176,7 @@ class CameraGeometry:
             return False
 
         return all(
-            [(self.pix_x == other.pix_x).all(), (self.pix_y == other.pix_y).all(),]
+            [(self.pix_x == other.pix_x).all(), (self.pix_y == other.pix_y).all()]
         )
 
     def guess_radius(self):
@@ -150,41 +192,59 @@ class CameraGeometry:
             (self.pix_x[border] - cx) ** 2 + (self.pix_y[border] - cy) ** 2
         ).mean()
 
-    def transform_to(self, frame):
-        """
-        Transform the pixel coordinates stored in this geometry
-        and the pixel and camera rotations to another camera coordinate frame.
+    def transform_to(self, frame: BaseCoordinateFrame) -> CG:
+        """Transform the pixel coordinates stored in this geometry and the pixel
+        and camera rotations to another camera coordinate frame.
+
+        Note that `geom.frame` must contain all the necessary attributes needed
+        to transform into the requested frame, i.e. if going from `CameraFrame`
+        to `TelescopeFrame`, it should contain a `focal_length` attribute.
 
         Parameters
         ----------
         frame: ctapipe.coordinates.CameraFrame
             The coordinate frame to transform to.
+
+        Returns
+        -------
+        CameraGeometry:
+            new instance in the requested Frame
         """
         if self.frame is None:
             self.frame = CameraFrame()
 
-        coord = SkyCoord(x=self.pix_x, y=self.pix_y, frame=self.frame)
+        coord = SkyCoord(self.pix_x, self.pix_y, frame=self.frame)
         trans = coord.transform_to(frame)
 
         # also transform the unit vectors, to get rotation / mirroring
-        uv = SkyCoord(x=[1, 0], y=[0, 1], unit=u.m, frame=self.frame)
+        uv = SkyCoord([1, 0], [0, 1], unit=self.pix_x.unit, frame=self.frame)
         uv_trans = uv.transform_to(frame)
-        rot = np.arctan2(uv_trans[0].y, uv_trans[1].y)
-        det = np.linalg.det([uv_trans.x.value, uv_trans.y.value])
 
-        cam_rotation = rot + det * self.cam_rotation
-        pix_rotation = rot + det * self.pix_rotation
+        # some trickery has to be done to deal with the fact that not all frames
+        # use the same x/y attributes. Therefore we get the component names, and
+        # access them by string:
+        frame_attrs = list(uv_trans.frame.get_representation_component_names().keys())
+        uv_x = getattr(uv_trans, frame_attrs[0])
+        uv_y = getattr(uv_trans, frame_attrs[1])
+        trans_x = getattr(trans, frame_attrs[0])
+        trans_y = getattr(trans, frame_attrs[1])
+
+        rot = np.arctan2(uv_y[0], uv_y[1])
+        det = np.linalg.det([uv_x.value, uv_y.value])
+
+        cam_rotation = rot - self.cam_rotation
+        pix_rotation = rot - self.pix_rotation
 
         return CameraGeometry(
             camera_name=self.camera_name,
             pix_id=self.pix_id,
-            pix_x=trans.x,
-            pix_y=trans.y,
-            pix_area=self.pix_area,
+            pix_x=trans_x,
+            pix_y=trans_y,
+            pix_area=self.guess_pixel_area(trans_x, trans_y, self.pix_type),
             pix_type=self.pix_type,
             pix_rotation=pix_rotation,
             cam_rotation=cam_rotation,
-            neighbors=None,
+            neighbors=self._neighbors,
             apply_derotation=False,
             frame=frame,
         )
@@ -193,8 +253,8 @@ class CameraGeometry:
         return hash(
             (
                 self.camera_name,
-                self.pix_x[0].to_value(u.m),
-                self.pix_y[0].to_value(u.m),
+                self.pix_x[0].value,
+                self.pix_y[0].value,
                 self.pix_type,
                 self.pix_rotation.deg,
             )
@@ -229,9 +289,9 @@ class CameraGeometry:
 
         dist = cls.guess_pixel_width(pix_x, pix_y)
 
-        if pix_type.startswith("hex"):
+        if pix_type == PixelShape.HEXAGON:
             area = 2 * np.sqrt(3) * (dist / 2) ** 2
-        elif pix_type.startswith("rect"):
+        elif pix_type == PixelShape.SQUARE:
             area = dist ** 2
         else:
             raise KeyError("unsupported pixel type")
@@ -241,17 +301,22 @@ class CameraGeometry:
     @lazyproperty
     def pixel_width(self):
         """
-        in-circle diameter for hexagons, edge width for square pixels
+        in-circle diameter for hexagons, edge width for square pixels,
+        diameter for circles.
 
         This is calculated from the pixel area.
         """
 
-        if self.pix_type.startswith("hex"):
+        if self.pix_type == PixelShape.HEXAGON:
             width = 2 * np.sqrt(self.pix_area / (2 * np.sqrt(3)))
-        elif self.pix_type.startswith("rect"):
+        elif self.pix_type == PixelShape.SQUARE:
             width = np.sqrt(self.pix_area)
+        elif self.pix_type == PixelShape.CIRCLE:
+            width = 2 * np.sqrt(self.pix_area / np.pi)
         else:
-            raise KeyError("unsupported pixel type")
+            raise NotImplementedError(
+                f"Cannot calculate pixel width for type {self.pix_type!r}"
+            )
 
         return width
 
@@ -271,17 +336,20 @@ class CameraGeometry:
         )
 
     @lazyproperty
-    def _pixel_circumferences(self):
+    def _pixel_circumradius(self):
         """ pixel circumference radius/radii based on pixel area and layout
-
         """
 
-        if self.pix_type.startswith("hex"):
-            circum_rad = np.sqrt(2.0 * self.pix_area / 3.0 / np.sqrt(3))
-        elif self.pix_type.startswith("rect"):
+        if self.pix_type == PixelShape.HEXAGON:
+            circum_rad = self.pixel_width / np.sqrt(3)
+        elif self.pix_type == PixelShape.SQUARE:
             circum_rad = np.sqrt(self.pix_area / 2.0)
+        elif self.pix_type == PixelShape.CIRCLE:
+            circum_rad = self.pixel_width / 2
         else:
-            raise KeyError("unsupported pixel type")
+            raise NotImplementedError(
+                "Cannot calculate pixel circumradius for type {self.pix_type!r}"
+            )
 
         return circum_rad
 
@@ -296,9 +364,7 @@ class CameraGeometry:
 
         """
 
-        pixel_centers = np.column_stack(
-            [self.pix_x.to_value(u.m), self.pix_y.to_value(u.m)]
-        )
+        pixel_centers = np.column_stack([self.pix_x.value, self.pix_y.value])
         return KDTree(pixel_centers)
 
     @lazyproperty
@@ -353,7 +419,7 @@ class CameraGeometry:
             [self.pix_id, self.pix_x, self.pix_y, self.pix_area],
             names=["pix_id", "pix_x", "pix_y", "pix_area"],
             meta=dict(
-                PIX_TYPE=self.pix_type,
+                PIX_TYPE=self.pix_type.value,
                 TAB_TYPE="ctapipe.instrument.CameraGeometry",
                 TAB_VER="1.1",
                 CAM_ID=self.camera_name,
@@ -396,7 +462,7 @@ class CameraGeometry:
 
     def __repr__(self):
         return (
-            "CameraGeometry(camera_name='{camera_name}', pix_type='{pix_type}', "
+            "CameraGeometry(camera_name='{camera_name}', pix_type={pix_type!r}, "
             "npix={npix}, cam_rot={camrot}, pix_rot={pixrot})"
         ).format(
             camera_name=self.camera_name,
@@ -437,7 +503,8 @@ class CameraGeometry:
         """
         neighbors = lil_matrix((self.n_pixels, self.n_pixels), dtype=bool)
 
-        if self.pix_type.startswith("hex"):
+        # assume circle pixels are also on a hex grid
+        if self.pix_type in (PixelShape.HEXAGON, PixelShape.CIRCLE):
             max_neighbors = 6
             # on a hexgrid, the closest pixel in the second circle is
             # the diameter of the hexagon plus the inradius away
@@ -487,19 +554,6 @@ class CameraGeometry:
 
         return neighbors.tocsr()
 
-    @lazyproperty
-    def neighbor_matrix_where(self):
-        """
-        Obtain a 2D array, where each row is [pixel index, one neighbour
-        of that pixel].
-
-        Returns
-        -------
-        ndarray
-        """
-        coo = self.neighbor_matrix_sparse.tocoo()
-        return np.column_stack([coo.row, coo.col])
-
     @lazyproperty
     def pixel_moment_matrix(self):
         """
@@ -684,9 +738,9 @@ class CameraGeometry:
             logger.warning(
                 " Method not implemented for cameras with varying pixel sizes"
             )
-
-        points_searched = np.dstack([x.to_value(u.m), y.to_value(u.m)])
-        circum_rad = self._pixel_circumferences[0].to_value(u.m)
+        unit = x.unit
+        points_searched = np.dstack([x.to_value(unit), y.to_value(unit)])
+        circum_rad = self._pixel_circumradius[0].to_value(unit)
         kdtree = self._kdtree
         dist, pix_indices = kdtree.query(
             points_searched, distance_upper_bound=circum_rad
@@ -718,13 +772,13 @@ class CameraGeometry:
                 # compare with inside pixel:
                 xprime = (
                     points_searched[0][index, 0]
-                    - self.pix_x[borderpix_index].to_value(u.m)
-                    + self.pix_x[insidepix_index].to_value(u.m)
+                    - self.pix_x[borderpix_index].to_value(unit)
+                    + self.pix_x[insidepix_index].to_value(unit)
                 )
                 yprime = (
                     points_searched[0][index, 1]
-                    - self.pix_y[borderpix_index].to_value(u.m)
-                    + self.pix_y[insidepix_index].to_value(u.m)
+                    - self.pix_y[borderpix_index].to_value(unit)
+                    + self.pix_y[insidepix_index].to_value(unit)
                 )
                 dist_check, index_check = kdtree.query(
                     [xprime, yprime], distance_upper_bound=circum_rad
@@ -745,16 +799,10 @@ class CameraGeometry:
 
     @staticmethod
     def simtel_shape_to_type(pixel_shape):
-        if pixel_shape == 1:
-            return "hexagonal", Angle(0, u.deg)
-
-        if pixel_shape == 2:
-            return "rectangular", Angle(0, u.deg)
-
-        if pixel_shape == 3:
-            return "hexagonal", Angle(30, u.deg)
-
-        raise ValueError(f"Unknown pixel_shape {pixel_shape}")
+        try:
+            return SIMTEL_PIXEL_SHAPES[pixel_shape]
+        except KeyError:
+            raise ValueError(f"Unknown pixel_shape {pixel_shape}") from None
 
 
 class UnknownPixelShapeWarning(UserWarning):
