@@ -1,5 +1,6 @@
 import warnings
 from gzip import GzipFile
+from io import BufferedReader
 from pathlib import Path
 
 import numpy as np
@@ -9,24 +10,29 @@ from astropy.time import Time
 from eventio.file_types import is_eventio
 from eventio.simtel.simtelfile import SimTelFile
 from traitlets import observe
-from io import BufferedReader
 
-from ..calib.camera.gainselection import ThresholdGainSelector
-from ..containers import EventAndMonDataContainer, EventType
-from ..core.traits import Bool, CaselessStrEnum
+from ..calib.camera.gainselection import GainSelector
+from ..containers import (
+    ArrayEventContainer,
+    EventType,
+    SimulationConfigContainer,
+    SimulatedCameraContainer,
+    SimulatedShowerContainer,
+)
+from ..coordinates import CameraFrame
+from ..core.traits import Bool, CaselessStrEnum, create_class_enum_trait
 from ..instrument import (
-    TelescopeDescription,
-    SubarrayDescription,
     CameraDescription,
     CameraGeometry,
     CameraReadout,
     OpticsDescription,
+    SubarrayDescription,
+    TelescopeDescription,
 )
 from ..instrument.camera import UnknownPixelShapeWarning
-from ..instrument.guess import guess_telescope, UNKNOWN_TELESCOPE
-from ..containers import MCHeaderContainer
-from .eventsource import EventSource
+from ..instrument.guess import UNKNOWN_TELESCOPE, guess_telescope
 from .datalevels import DataLevel
+from .eventsource import EventSource
 
 X_MAX_UNIT = u.g / (u.cm ** 2)
 
@@ -53,7 +59,7 @@ def parse_simtel_time(simtel_time):
     )
 
 
-def build_camera(cam_settings, pixel_settings, telescope):
+def build_camera(cam_settings, pixel_settings, telescope, frame):
     pixel_shape = cam_settings["pixel_shape"][0]
     try:
         pix_type, pix_rotation = CameraGeometry.simtel_shape_to_type(pixel_shape)
@@ -75,12 +81,15 @@ def build_camera(cam_settings, pixel_settings, telescope):
         pix_rotation=pix_rotation,
         cam_rotation=-Angle(cam_settings["cam_rot"], u.rad),
         apply_derotation=True,
+        frame=frame,
     )
     readout = CameraReadout(
         telescope.camera_name,
         sampling_rate=u.Quantity(1 / pixel_settings["time_slice"], u.GHz),
         reference_pulse_shape=pixel_settings["ref_shape"].astype("float64", copy=False),
-        reference_pulse_sample_width=u.Quantity(pixel_settings["ref_step"], u.ns),
+        reference_pulse_sample_width=u.Quantity(
+            pixel_settings["ref_step"], u.ns, dtype="float64"
+        ),
     )
 
     return CameraDescription(
@@ -122,7 +131,7 @@ def apply_simtel_r1_calibration(r0_waveforms, pedestal, dc_to_pe, gain_selector)
         Shape: (n_pixels)
     """
     n_channels, n_pixels, n_samples = r0_waveforms.shape
-    ped = pedestal[..., np.newaxis] / n_samples
+    ped = pedestal[..., np.newaxis]
     gain = dc_to_pe[..., np.newaxis]
     r1_waveforms = (r0_waveforms - ped) * gain
     if n_channels == 1:
@@ -135,6 +144,8 @@ def apply_simtel_r1_calibration(r0_waveforms, pedestal, dc_to_pe, gain_selector)
 
 
 class SimTelEventSource(EventSource):
+    """ Read events from a SimTelArray data file (in EventIO format)."""
+
     skip_calibration_events = Bool(True, help="Skip calibration events").tag(
         config=True
     )
@@ -160,9 +171,11 @@ class SimTelEventSource(EventSource):
         ),
     ).tag(config=True)
 
-    def __init__(
-        self, input_url, config=None, parent=None, gain_selector=None, **kwargs
-    ):
+    gain_selector_type = create_class_enum_trait(
+        base_class=GainSelector, default_value="ThresholdGainSelector"
+    ).tag(config=True)
+
+    def __init__(self, input_url=None, config=None, parent=None, **kwargs):
         """
         EventSource for simtelarray files using the pyeventio library.
 
@@ -181,6 +194,7 @@ class SimTelEventSource(EventSource):
         kwargs
         """
         super().__init__(input_url=input_url, config=config, parent=parent, **kwargs)
+
         self._camera_cache = {}
 
         self.file_ = SimTelFile(
@@ -195,15 +209,13 @@ class SimTelEventSource(EventSource):
         self._subarray_info = self.prepare_subarray_info(
             self.file_.telescope_descriptions, self.file_.header
         )
-        self._mc_header = self._parse_mc_header()
+        self._simulation_config = self._parse_simulation_header()
         self.start_pos = self.file_.tell()
 
-        # Waveforms from simtelarray have both gain channels
-        # Gain selection is performed by this EventSource to produce R1 waveforms
-        if gain_selector is None:
-            gain_selector = ThresholdGainSelector(parent=self)
-
-        self.gain_selector = gain_selector
+        self.gain_selector = GainSelector.from_name(
+            self.gain_selector_type, parent=self
+        )
+        self.log.debug(f"Using gain selector {self.gain_selector}")
 
     @observe("allowed_tels")
     def _observe_allowed_tels(self, change):
@@ -224,15 +236,16 @@ class SimTelEventSource(EventSource):
 
     @property
     def datalevels(self):
-        return (DataLevel.R0, DataLevel.R1, DataLevel.DL0)
+        return (DataLevel.R0, DataLevel.R1)
 
     @property
-    def obs_id(self):
-        return self.file_.header["run"]
+    def obs_ids(self):
+        # ToDo: This does not support merged simtel files!
+        return [self.file_.header["run"]]
 
     @property
-    def mc_header(self):
-        return self._mc_header
+    def simulation_config(self) -> SimulationConfigContainer:
+        return self._simulation_config
 
     @property
     def is_stream(self):
@@ -283,11 +296,6 @@ class SimTelEventSource(EventSource):
             except ValueError:
                 telescope = UNKNOWN_TELESCOPE
 
-            camera = self._camera_cache.get(telescope.camera_name)
-            if camera is None:
-                camera = build_camera(cam_settings, pixel_settings, telescope)
-                self._camera_cache[telescope.camera_name] = camera
-
             optics = OpticsDescription(
                 name=telescope.name,
                 num_mirrors=telescope.n_mirrors,
@@ -296,6 +304,16 @@ class SimTelEventSource(EventSource):
                 num_mirror_tiles=cam_settings["n_mirrors"],
             )
 
+            camera = self._camera_cache.get(telescope.camera_name)
+            if camera is None:
+                camera = build_camera(
+                    cam_settings,
+                    pixel_settings,
+                    telescope,
+                    frame=CameraFrame(focal_length=optics.equivalent_focal_length),
+                )
+                self._camera_cache[telescope.camera_name] = camera
+
             tel_descriptions[tel_id] = TelescopeDescription(
                 name=telescope.name,
                 tel_type=telescope.type,
@@ -335,11 +353,10 @@ class SimTelEventSource(EventSource):
             warnings.warn(msg)
 
     def _generate_events(self):
-        data = EventAndMonDataContainer()
+        data = ArrayEventContainer()
         data.meta["origin"] = "hessio"
         data.meta["input_url"] = self.input_url
         data.meta["max_events"] = self.max_events
-        data.mcheader = self._mc_header
 
         self._fill_array_pointing(data)
 
@@ -347,18 +364,14 @@ class SimTelEventSource(EventSource):
 
             event_id = array_event.get("event_id", -1)
             obs_id = self.file_.header["run"]
-            tels_with_data = set(array_event["telescope_events"].keys())
             data.count = counter
             data.index.obs_id = obs_id
             data.index.event_id = event_id
-            data.r0.tels_with_data = tels_with_data
-            data.r1.tels_with_data = tels_with_data
-            data.dl0.tels_with_data = tels_with_data
 
             self._fill_trigger_info(data, array_event)
 
             if data.trigger.event_type == EventType.SUBARRAY:
-                self._fill_mc_event_information(data, array_event)
+                self._fill_simulated_event_information(data, array_event)
 
             # this should be done in a nicer way to not re-allocate the
             # data each time (right now it's just deleted and garbage
@@ -367,62 +380,72 @@ class SimTelEventSource(EventSource):
             data.r1.tel.clear()
             data.dl0.tel.clear()
             data.dl1.tel.clear()
-            data.mc.tel.clear()
             data.pointing.tel.clear()
+            data.simulation.tel.clear()
 
             telescope_events = array_event["telescope_events"]
             tracking_positions = array_event["tracking_positions"]
+
             for tel_id, telescope_event in telescope_events.items():
                 adc_samples = telescope_event.get("adc_samples")
                 if adc_samples is None:
                     adc_samples = telescope_event["adc_sums"][:, :, np.newaxis]
-                n_gains, n_pixels, n_samples = adc_samples.shape
 
-                mc = data.mc.tel[tel_id]
-                mc.dc_to_pe = array_event["laser_calibrations"][tel_id]["calib"]
-                mc.pedestal = array_event["camera_monitorings"][tel_id]["pedestal"]
-                mc.true_image = (
+                n_gains, n_pixels, n_samples = adc_samples.shape
+                true_image = (
                     array_event.get("photoelectrons", {})
                     .get(tel_id - 1, {})
                     .get("photoelectrons", np.zeros(n_pixels, dtype="float32"))
                 )
 
+                data.simulation.tel[tel_id] = SimulatedCameraContainer(
+                    true_image=true_image
+                )
+
                 self._fill_event_pointing(
-                    data.pointing.tel[tel_id], mc, tracking_positions[tel_id],
+                    data.pointing.tel[tel_id], tracking_positions[tel_id]
                 )
 
                 r0 = data.r0.tel[tel_id]
                 r1 = data.r1.tel[tel_id]
                 r0.waveform = adc_samples
+
+                mon = array_event["camera_monitorings"][tel_id]
+                pedestal = mon["pedestal"] / mon["n_ped_slices"]
+                dc_to_pe = array_event["laser_calibrations"][tel_id]["calib"]
+                # todo: store pedestal and dc_to_pe somewhere?
+                #
                 r1.waveform, r1.selected_gain_channel = apply_simtel_r1_calibration(
-                    adc_samples, mc.pedestal, mc.dc_to_pe, self.gain_selector
+                    adc_samples, pedestal, dc_to_pe, self.gain_selector
                 )
 
-                pixel_lists = telescope_event["pixel_lists"]
-                r0.num_trig_pix = pixel_lists.get(0, {"pixels": 0})["pixels"]
-                if r0.num_trig_pix > 0:
-                    r0.trig_pix_id = pixel_lists[0]["pixel_list"]
+                # get time_shift from laser calibration
+                time_calib = array_event["laser_calibrations"][tel_id]["tm_calib"]
+                pix_index = np.arange(n_pixels)
+
+                dl1_calib = data.calibration.tel[tel_id].dl1
+                dl1_calib.time_shift = time_calib[r1.selected_gain_channel, pix_index]
 
             yield data
 
     @staticmethod
-    def _fill_event_pointing(pointing, mc, tracking_position):
-        mc.azimuth_raw = tracking_position["azimuth_raw"]
-        mc.altitude_raw = tracking_position["altitude_raw"]
-        mc.azimuth_cor = tracking_position.get("azimuth_cor", np.nan)
-        mc.altitude_cor = tracking_position.get("altitude_cor", np.nan)
+    def _fill_event_pointing(pointing, tracking_position):
+        azimuth_raw = tracking_position["azimuth_raw"]
+        altitude_raw = tracking_position["altitude_raw"]
+        azimuth_cor = tracking_position.get("azimuth_cor", np.nan)
+        altitude_cor = tracking_position.get("altitude_cor", np.nan)
 
         # take pointing corrected position if available
-        if np.isnan(mc.azimuth_cor):
-            pointing.azimuth = u.Quantity(mc.azimuth_raw, u.rad)
+        if np.isnan(azimuth_cor):
+            pointing.azimuth = u.Quantity(azimuth_raw, u.rad)
         else:
-            pointing.azimuth = u.Quantity(mc.azimuth_cor, u.rad)
+            pointing.azimuth = u.Quantity(azimuth_cor, u.rad)
 
         # take pointing corrected position if available
-        if np.isnan(mc.altitude_cor):
-            pointing.altitude = u.Quantity(mc.altitude_raw, u.rad)
+        if np.isnan(altitude_cor):
+            pointing.altitude = u.Quantity(altitude_raw, u.rad)
         else:
-            pointing.altitude = u.Quantity(mc.altitude_cor, u.rad)
+            pointing.altitude = u.Quantity(altitude_cor, u.rad)
 
     @staticmethod
     def _fill_trigger_info(data, array_event):
@@ -447,7 +470,17 @@ class SimTelEventSource(EventSource):
             trigger["triggered_telescopes"], trigger["trigger_times"]
         ):
             # time is relative to central trigger in nano seconds
-            data.trigger.tel[tel_id].time = data.trigger.time + u.Quantity(time, u.ns)
+            trigger = data.trigger.tel[tel_id]
+            trigger.time = data.trigger.time + u.Quantity(time, u.ns)
+
+            # triggered pixel info
+            tel_event = array_event["telescope_events"].get(tel_id)
+            if tel_event:
+                # code 0 = trigger pixels
+                pixel_list = tel_event["pixel_lists"].get(0)
+                if pixel_list:
+                    trigger.n_trigger_pixels = pixel_list["pixels"]
+                    trigger.trigger_pixels = pixel_list["pixel_list"]
 
     def _fill_array_pointing(self, data):
         if self.file_.header["tracking_mode"] == 0:
@@ -459,17 +492,17 @@ class SimTelEventSource(EventSource):
             data.pointing.array_ra = u.Quantity(ra, u.rad)
             data.pointing.array_dec = u.Quantity(dec, u.rad)
 
-    def _parse_mc_header(self):
+    def _parse_simulation_header(self):
         mc_run_head = self.file_.mc_run_headers[-1]
 
-        return MCHeaderContainer(
+        return SimulationConfigContainer(
             corsika_version=mc_run_head["shower_prog_vers"],
             simtel_version=mc_run_head["detector_prog_vers"],
             energy_range_min=mc_run_head["E_range"][0] * u.TeV,
             energy_range_max=mc_run_head["E_range"][1] * u.TeV,
             prod_site_B_total=mc_run_head["B_total"] * u.uT,
-            prod_site_B_declination=Angle(mc_run_head["B_declination"], u.rad,),
-            prod_site_B_inclination=Angle(mc_run_head["B_inclination"], u.rad,),
+            prod_site_B_declination=Angle(mc_run_head["B_declination"], u.rad),
+            prod_site_B_inclination=Angle(mc_run_head["B_inclination"], u.rad),
             prod_site_alt=mc_run_head["obsheight"] * u.m,
             spectral_index=mc_run_head["spectral_index"],
             shower_prog_start=mc_run_head["shower_prog_start"],
@@ -498,20 +531,20 @@ class SimTelEventSource(EventSource):
             corsika_wlen_max=mc_run_head["corsika_wlen_max"] * u.nm,
             corsika_low_E_detail=mc_run_head["corsika_low_E_detail"],
             corsika_high_E_detail=mc_run_head["corsika_high_E_detail"],
-            run_array_direction=Angle(self.file_.header["direction"] * u.rad),
         )
 
     @staticmethod
-    def _fill_mc_event_information(data, array_event):
+    def _fill_simulated_event_information(data, array_event):
         mc_event = array_event["mc_event"]
         mc_shower = array_event["mc_shower"]
 
-        data.mc.energy = u.Quantity(mc_shower["energy"], u.TeV)
-        data.mc.alt = Angle(mc_shower["altitude"], u.rad)
-        data.mc.az = Angle(mc_shower["azimuth"], u.rad)
-        data.mc.core_x = u.Quantity(mc_event["xcore"], u.m)
-        data.mc.core_y = u.Quantity(mc_event["ycore"], u.m)
-        first_int = u.Quantity(mc_shower["h_first_int"], u.m)
-        data.mc.h_first_int = first_int
-        data.mc.x_max = u.Quantity(mc_shower["xmax"], X_MAX_UNIT)
-        data.mc.shower_primary_id = mc_shower["primary_id"]
+        data.simulation.shower = SimulatedShowerContainer(
+            energy=u.Quantity(mc_shower["energy"], u.TeV),
+            alt=Angle(mc_shower["altitude"], u.rad),
+            az=Angle(mc_shower["azimuth"], u.rad),
+            core_x=u.Quantity(mc_event["xcore"], u.m),
+            core_y=u.Quantity(mc_event["ycore"], u.m),
+            h_first_int=u.Quantity(mc_shower["h_first_int"], u.m),
+            x_max=u.Quantity(mc_shower["xmax"], X_MAX_UNIT),
+            shower_primary_id=mc_shower["primary_id"],
+        )
