@@ -6,12 +6,13 @@ from astropy.coordinates import SkyCoord
 from ctapipe.coordinates import NominalFrame, AltAz, CameraFrame
 from ctapipe.containers import HillasParametersContainer
 
-from ctapipe.io import event_source
+from ctapipe.io import EventSource
 
 from ctapipe.utils import get_dataset_path
 
 from ctapipe.image.cleaning import tailcuts_clean
 from ctapipe.image.hillas import hillas_parameters, HillasParameterizationError
+from ctapipe.calib import CameraCalibrator
 
 
 def test_intersect():
@@ -241,19 +242,23 @@ def test_reconstruction():
 
     fit = HillasIntersection()
 
-    source = event_source(filename, max_events=10)
+    source = EventSource(filename, max_events=10)
+    calib = CameraCalibrator(source.subarray)
 
     horizon_frame = AltAz()
 
     reconstructed_events = 0
 
     for event in source:
-        array_pointing = SkyCoord(az=event.mc.az, alt=event.mc.alt, frame=horizon_frame)
+        calib(event)
+
+        mc = event.simulation.shower
+        array_pointing = SkyCoord(az=mc.az, alt=mc.alt, frame=horizon_frame)
 
         hillas_dict = {}
         telescope_pointings = {}
 
-        for tel_id in event.dl0.tels_with_data:
+        for tel_id, dl1 in event.dl1.tel.items():
 
             geom = source.subarray.tel[tel_id].camera.geometry
 
@@ -262,15 +267,13 @@ def test_reconstruction():
                 az=event.pointing.tel[tel_id].azimuth,
                 frame=horizon_frame,
             )
-            pmt_signal = event.r0.tel[tel_id].waveform[0].sum(axis=1)
 
             mask = tailcuts_clean(
-                geom, pmt_signal, picture_thresh=10.0, boundary_thresh=5.0
+                geom, dl1.image, picture_thresh=10.0, boundary_thresh=5.0
             )
-            pmt_signal[mask == 0] = 0
 
             try:
-                moments = hillas_parameters(geom, pmt_signal)
+                moments = hillas_parameters(geom[mask], dl1.image[mask])
                 hillas_dict[tel_id] = moments
             except HillasParameterizationError as e:
                 print(e)
@@ -287,7 +290,7 @@ def test_reconstruction():
         )
 
         print(fit_result)
-        print(event.mc.core_x, event.mc.core_y)
+        print(event.simulation.shower.core_x, event.simulation.shower.core_y)
         fit_result.alt.to(u.deg)
         fit_result.az.to(u.deg)
         fit_result.core_x.to(u.m)
